@@ -17,6 +17,7 @@ BEGIN_EVENT_TABLE(CUpdateDialog, wxDialogEx)
 EVT_BUTTON(XRCID("ID_INSTALL"), CUpdateDialog::OnInstall)
 EVT_TIMER(wxID_ANY, CUpdateDialog::OnTimer)
 EVT_HYPERLINK(XRCID("ID_SHOW_DETAILS"), CUpdateDialog::ShowDetails)
+EVT_HYPERLINK(XRCID("ID_SHOW_DETAILS_DL"), CUpdateDialog::ShowDetailsDl)
 EVT_HYPERLINK(XRCID("ID_RETRY"), CUpdateDialog::Retry)
 EVT_HYPERLINK(XRCID("ID_DOWNLOAD_RETRY"), CUpdateDialog::Retry)
 EVT_BUTTON(XRCID("ID_DEBUGLOG"), CUpdateDialog::OnDebugLog)
@@ -54,20 +55,20 @@ bool CUpdateDialog::IsRunning()
 int CUpdateDialog::ShowModal()
 {
 	wxString version(PACKAGE_VERSION, wxConvLocal);
-	if (version.empty() || version[0] < '0' || version[0] > '9')
-	{
+	if (version.empty() || version[0] < '0' || version[0] > '9') {
 		wxMessageBoxEx(_("Executable contains no version info, cannot check for updates."), _("Check for updates failed"), wxICON_ERROR, parent_);
 		return wxID_CANCEL;
 	}
 
-	if (!Load(parent_, _T("ID_UPDATE_DIALOG")))
+	if (!Load(parent_, _T("ID_UPDATE_DIALOG"))) {
 		return wxID_CANCEL;
+	}
 
 	LoadPanel(_T("ID_CHECKING_PANEL"));
 	LoadPanel(_T("ID_FAILURE_PANEL"));
 	LoadPanel(_T("ID_NEWVERSION_PANEL"));
 	LoadPanel(_T("ID_LATEST_PANEL"));
-	if( panels_.size() != 4 ) {
+	if (panels_.size() != 4) {
 		return wxID_CANCEL;
 	}
 
@@ -84,6 +85,7 @@ int CUpdateDialog::ShowModal()
 	Wrap();
 
 	xrc_call(*this, "ID_DETAILS", &wxTextCtrl::Hide);
+	xrc_call(*this, "ID_DETAILS_DL", &wxTextCtrl::Hide);
 
 	UpdaterState s = updater_.GetState();
 	UpdaterStateChanged( s, updater_.AvailableBuild() );
@@ -258,6 +260,9 @@ void CUpdateDialog::UpdaterStateChanged( UpdaterState s, build const& v )
 		bool dlfail = s == UpdaterState::newversion && !v.url_.empty();
 		XRCCTRL(*this, "ID_DOWNLOAD_FAIL", wxStaticText)->Show(dlfail);
 		XRCCTRL(*this, "ID_DOWNLOAD_RETRY", wxHyperlinkCtrl)->Show(dlfail);
+		XRCCTRL(*this, "ID_SHOW_DETAILS_DL", wxHyperlinkCtrl)->Show(dlfail);
+
+		XRCCTRL(*this, "ID_DETAILS_DL", wxTextCtrl)->ChangeValue(updater_.GetLog());
 
 		XRCCTRL(*this, "ID_NEWVERSION_WEBSITE_TEXT", wxStaticText)->Show(manual && !dlfail);
 		XRCCTRL(*this, "ID_NEWVERSION_WEBSITE_TEXT_DLFAIL", wxStaticText)->Show(manual && dlfail);
@@ -316,6 +321,15 @@ void CUpdateDialog::ShowDetails(wxHyperlinkEvent&)
 	panels_[pagenames::failed]->Layout();
 }
 
+void CUpdateDialog::ShowDetailsDl(wxHyperlinkEvent&)
+{
+	XRCCTRL(*this, "ID_SHOW_DETAILS_DL", wxHyperlinkCtrl)->Hide();
+	XRCCTRL(*this, "ID_DETAILS_DL", wxTextCtrl)->Show();
+	XRCCTRL(*this, "ID_DETAILS_DL", wxTextCtrl)->SetMinSize(wxSize(-1, 200));
+
+	panels_[pagenames::newversion]->Layout();
+}
+
 void CUpdateDialog::Retry(wxHyperlinkEvent&)
 {
 	updater_.RunIfNeeded();
